///////////////////////////////////////////////////////////////////
// This software is copyrighted by Optical Communications Group. //
// University of Valladolid (UVa)                                //
//                                                               //
// WRON simulator                                                //
// has been developed by Optical Communications Group.           //
// University of Valladolid (UVa)                                //
// Contact e-mail: rduran@tel.uva.es,natalia.fernandez@tel.uva.es //
//                                                               //
// Escuela Técnica Superior de Ingenieros de Telecomunicación    //
// Campus Miguel Delibes                                         //
// Universidad de Valladolid                                     //
// Paseo de Belén, 15                                            //
// 47011 Valladolid, Spain                                       //
//                                                               //
// This file is part of CHRON simulator,                         //
// an optical network simulation tool                            //
// based on OMNeT++/OMNEST discrete-event simulation framework.  //
//                                                               //
// Authoring:                                                    //
// Natalia Fernández Sordo (natalia.fernandez@tel.uva.es)        //
// Ramón J. Durán Barroso (rduran@tel.uva.es)                    //
// Ignacio de Miguel Jiménez (ignacio.miguel@tel.uva.es)         //
///////////////////////////////////////////////////////////////////

#include "cTopologyextension.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <deque>
#include <algorithm>
#include <sstream>
#include <omnetpp.h>

//#include "patternmatcher.h"



USING_NAMESPACE

Register_Class(cTopologyExtension);


cTopologyExtension::LinkInExtension *cTopologyExtension::NodeExtension::getLinkIn(int i)
{
    if (i<0 || i>=num_in_links)
        throw cRuntimeError("cTopologyExtension::NodeExtension::getLinkIn: invalid link index %d", i);
    return (cTopologyExtension::LinkInExtension *)in_links[i];
}

cTopologyExtension::LinkOutExtension *cTopologyExtension::NodeExtension::getLinkOut(int i)
{
    if (i<0 || i>=num_out_links)
        throw cRuntimeError("cTopologyExtension::NodeExtension::getLinkOut: invalid index %d", i);
    return (cTopologyExtension::LinkOutExtension *)(out_links+i);
}

//----

cTopologyExtension::cTopologyExtension(const char *name) : cTopology(name)
{
    num_nodes = 0;
    nodev = NULL;
}

cTopologyExtension::cTopologyExtension(const cTopologyExtension& topo) : cTopology()
{
    nodev = NULL;
    setName(topo.getName());
    cTopologyExtension::operator=(topo);

}

cTopologyExtension::~cTopologyExtension()
{
    clear();
}

std::string cTopologyExtension::info() const
{
    std::stringstream out;
    out << "n=" << num_nodes;
    return out.str();
}

void cTopologyExtension::parsimPack(cCommBuffer *buffer)
{
    throw cRuntimeError(this,"parsimPack() not implemented");
}

void cTopologyExtension::parsimUnpack(cCommBuffer *buffer)
{
    throw cRuntimeError(this,"parsimUnpack() not implemented");
}

cTopologyExtension& cTopologyExtension::operator=(const cTopologyExtension&)
{
    throw cRuntimeError(this,"operator= not implemented yet");
}

void cTopologyExtension::clear()
{
    for (int i=0; i<num_nodes; i++)
    {
        delete [] nodev[i].in_links;
        delete [] nodev[i].out_links;
    }
    delete [] nodev;

    num_nodes = 0;

    nodev = NULL;
}

//---
/*
static bool selectByModulePath(cModule *mod, void *data)
{
    // actually, this is selectByModuleFullPathPattern()
    const std::vector<std::string>& v = *(const std::vector<std::string> *)data;
    std::string path = mod->getFullPath();
    for (int i=0; i<(int)v.size(); i++)
        if (PatternMatcher(v[i].c_str(), true, true, true).matches(path.c_str()))
            return true;
    return false;
}*/

static bool selectByNedTypeName(cModule *mod, void *data)
{
    const std::vector<std::string>& v = *(const std::vector<std::string> *)data;
    return std::find(v.begin(), v.end(), mod->getNedTypeName()) != v.end();
}

static bool selectByProperty(cModule *mod, void *data)
{
    struct ParamData {const char *name; const char *value;};
    ParamData *d = (ParamData *)data;
    cProperty *prop = mod->getProperties()->get(d->name);
    if (!prop)
        return false;
    const char *value = prop->getValue(cProperty::DEFAULTKEY, 0);
    if (d->value)
        return opp_strcmp(value, d->value)==0;
    else
        return opp_strcmp(value, "false")!=0;
}

static bool selectByParameter(cModule *mod, void *data)
{
    struct PropertyData{const char *name; const char *value;};
    PropertyData *d = (PropertyData *)data;
    return mod->hasPar(d->name) && (d->value==NULL || mod->par(d->name).str()==std::string(d->value));
}

//---

void cTopologyExtension::extractByModulePath(const std::vector<std::string>& fullPathPatterns)
{
    //extractFromNetwork(selectByModulePath, (void *)&fullPathPatterns);
}

void cTopologyExtension::extractByNedTypeName(const std::vector<std::string>& nedTypeNames)
{
    extractFromNetwork(selectByNedTypeName, (void *)&nedTypeNames);
}

void cTopologyExtension::extractByProperty(const char *propertyName, const char *value)
{
    struct {const char *name; const char *value;} data = {propertyName, value};
    extractFromNetwork(selectByProperty, (void *)&data);
}

void cTopologyExtension::extractByParameter(const char *paramName, const char *paramValue)
{
    struct {const char *name; const char *value;} data = {paramName, paramValue};
    extractFromNetwork(selectByParameter, (void *)&data);
}

//---

static bool selectByPredicate(cModule *mod, void *data)
{
    cTopologyExtension::PredicateExtension *predicate = (cTopologyExtension::PredicateExtension *)data;
    return predicate->matches(mod);
}

void cTopologyExtension::extractFromNetwork(PredicateExtension *predicate)
{
    extractFromNetwork(selectByPredicate, (void *)predicate);
}

void cTopologyExtension::extractFromNetwork(bool (*selfunc)(cModule *,void *), void *data)
{
    clear();

    NodeExtension *temp_nodev = new NodeExtension[simulation.getLastModuleId()];

    // Loop through all modules and find those which have the required
    // parameter with the (optionally) required value.
    int k=0;
    for (int mod_id=0; mod_id<=simulation.getLastModuleId(); mod_id++)
    {
        cModule *mod = simulation.getModule(mod_id);
        if (mod && selfunc(mod,data))
        {
            // ith module is OK, insert into nodev[]
            temp_nodev[k].module_id = mod_id;
            temp_nodev[k].wgt = 0;
            temp_nodev[k].enabl = true;

            // init auxiliary variables
            temp_nodev[k].known = 0;
            temp_nodev[k].dist = INFINITY;
            //temp_nodev[k].out_path.resize(0);
            temp_nodev[k].out_path=NULL;

            // create in_links[] arrays (big enough...)
            temp_nodev[k].num_in_links = 0;
            temp_nodev[k].in_links = new cTopologyExtension::LinkExtension *[mod->gateCount()];
            k++;
        }
    }
    num_nodes = k;

    nodev = new NodeExtension[num_nodes];
    memcpy(nodev, temp_nodev, num_nodes*sizeof(NodeExtension));
    delete [] temp_nodev;


    // Discover out neighbors too.
    for (int k=0; k<num_nodes; k++)
    {
        // Loop through all its gates and find those which come
        // from or go to modules included in the topology.

	cModule *mod = simulation.getModule(nodev[k].module_id);
        cTopologyExtension::LinkExtension *temp_out_links = new cTopologyExtension::LinkExtension[mod->gateCount()];

        int n_out=0;
        for (cModule::GateIterator i(mod); !i.end(); i++)
        {
            cGate *gate = i();
            if (gate->getType()!=cGate::OUTPUT)
                continue;

            // follow path
            cGate *src_gate = gate;
            do {
                gate = gate->getNextGate();
            }
            while(gate && !selfunc(gate->getOwnerModule(),data));

            // if we arrived at a module in the topology, record it.
            if (gate)
            {
                temp_out_links[n_out].src_node = nodev+k;
                temp_out_links[n_out].src_gate = src_gate->getId();
                temp_out_links[n_out].dest_node = getNodeFor(gate->getOwnerModule());
                temp_out_links[n_out].dest_gate = gate->getId();
                temp_out_links[n_out].wgt = 1.0;
                temp_out_links[n_out].enabl = true;
                n_out++;

            }
        }
        nodev[k].num_out_links = n_out;

        nodev[k].out_links = new cTopologyExtension::LinkExtension[n_out];

        memcpy(nodev[k].out_links, temp_out_links, n_out*sizeof(cTopologyExtension::LinkExtension));
        delete [] temp_out_links;
    }

    // fill in_links[] arrays
    for (int k=0; k<num_nodes; k++)
    {
        for (int l=0; l<nodev[k].num_out_links; l++)
        {
            cTopologyExtension::LinkExtension *link = &nodev[k].out_links[l];

            link->dest_node->in_links[link->dest_node->num_in_links++] = link;
        }
    }
}

cTopologyExtension::NodeExtension *cTopologyExtension::getNode(int i)
{
    if (i<0 || i>=num_nodes)
        throw cRuntimeError(this,"invalid node index %d",i);
    return nodev+i;
}

cTopologyExtension::NodeExtension *cTopologyExtension::getNodeFor(cModule *mod)
{
    // binary search can be done because nodev[] is ordered

    int lo, up, index;
    for ( lo=0, up=num_nodes, index=(lo+up)/2;
          lo<index;
          index=(lo+up)/2 )
    {
        // cycle invariant: nodev[lo].mod_id <= mod->getId() < nodev[up].mod_id
        if (mod->getId() < nodev[index].module_id)
             up = index;
        else
             lo = index;
    }
    return (mod->getId() == nodev[index].module_id) ? nodev+index : NULL;
}

void cTopologyExtension::calculateUnweightedSingleShortestPathsTo(NodeExtension *_target)
{
    // multiple paths not supported :-(

    if (!_target)
        throw cRuntimeError(this,"..ShortestPathTo(): target node is NULL");
    target = _target;

    for (int i=0; i<num_nodes; i++)
    {
       nodev[i].known = false;   // not really needed for unweighted
       nodev[i].dist = INFINITY;
       nodev[i].out_path=NULL;
    }
    target->dist = 0;

    std::deque<NodeExtension*> q;

    q.push_back(target);

    while (!q.empty())
    {
       NodeExtension *v = q.front();
       q.pop_front();

       // for each w adjacent to v...
       for (int i=0; i<(v->num_in_links); i++)
       {
           if (!(v->in_links[i]->enabl)) continue;
           NodeExtension *w = v->in_links[i]->src_node;
           if (!w->enabl) continue;
           if (w->dist == INFINITY)
           {
               w->dist = v->dist + 1;
               w->out_path = v->in_links[i];
               q.push_back(w);
           }
       }
    }
}




cTopologyExtension::NodeExtension *cTopologyExtension::NodeExtension::paths_m(int i)
{

    return (cTopologyExtension::NodeExtension *)(node_paths_m[(i%num_paths_m)]);
}

cTopologyExtension::LinkOutExtension *cTopologyExtension::NodeExtension::getPath(int i)
{
	return (cTopologyExtension::LinkOutExtension *)out_path;
}

void cTopologyExtension::calculateUnweightedMultipleShortestPathsTo(NodeExtension *_target)
{
	if (!_target)
	        throw cRuntimeError(this,"..ShortestPathTo(): target node is NULL");
	    target = _target;
	    for (int i=0; i<num_nodes; i++)
		{
			nodev[i].known = false;   // not really needed for unweighted
			nodev[i].dist = INFINITY;
			nodev[i].out_path=NULL;
			if(nodev[i].num_paths_m>0)
				nodev[i].node_paths_m.resize(0);
			nodev[i].num_paths_m = 0;
		}
		target->dist = 0;

	    std::deque<NodeExtension*> q;

	    q.push_back( target );
	    while (!q.empty())
	    {
	    	NodeExtension *v = q.front();
	    	q.pop_front();


	    	// for each w adjacent to v...
	    	for (int i=0; i<v->num_in_links; i++)
	    	{

	    		if (!(v->in_links[i]->enabl))
				{
					continue;
				}

				NodeExtension *w = v->in_links[i]->src_node;

				if (!(w->enabl))
				{
					continue;
				}

				if (w==target)
				{
					 continue;
				}
	    		else if (w->dist == INFINITY)
			    {
					 w->dist = v->dist + 1;
					 w->out_path=v->in_links[i];
					 w->num_paths_m=v->num_paths_m;
					 if(v==target)w->num_paths_m=1;
					 w->node_paths_m.resize(w->num_paths_m);
					 for(int j=0;j<=(w->num_paths_m-1);j++)w->node_paths_m[j]=v;
					 q.push_back(w);
			    }
	    		else if((w->node_paths_m[(w->num_paths_m)-1]!=v))
	    		{
	    			if(w->dist > (v->dist +1))
	    			{
	    				w->dist =v->dist+1;
	    				w->out_path=v->in_links[i];
	    				q.push_back(w);
	    			}
	    			else if(w->dist == (v->dist+1))
	    			{
	    				w->out_path=v->in_links[i];
	    				int previous_paths=w->num_paths_m;
	    				w->num_paths_m+=v->num_paths_m;
	    				w->node_paths_m.resize(w->num_paths_m);
	    				w->node_paths_m[w->num_paths_m-1]=v;
	    				for(int j=previous_paths;j<=(w->num_paths_m)-1;j++)w->node_paths_m[j]=v;
	    				q.push_back(w);
	    			}
	    		}
	    	}
	    }

}




void cTopologyExtension::calculateWeightedMultipleShortestPathsTo(NodeExtension *_target)
{
	int j,old,counter,r;
	double distv, distw;
	double current_distance;
	int num_links_to_node;
	int auxiliar;
	int paths_accumulated_previous_node;
	LinkExtension *aux_output;



	if (!_target)
	        throw cRuntimeError(this,"..ShortestPathTo(): target node is NULL");
	target = _target;

	for (int i=0; i<num_nodes; i++)
	{
		nodev[i].known = false;   // not really needed for unweighted
		nodev[i].dist = INFINITY;
		nodev[i].out_path=NULL;
		nodev[i].array_links_path.resize(0); //Vector to save links
		if(nodev[i].num_paths_m>0)nodev[i].node_paths_m.resize(0);
		nodev[i].num_paths_m = 0;
	}

	target->dist = 0.0;

    std::deque<NodeExtension*> q;

    q.push_back( target );

    while (!q.empty())
    {
       		NodeExtension *v = q.front();
       		q.pop_front();


       		for (int i=0; i<v->num_in_links; i++)
       		{
          		if (!(v->in_links[i]->enabl))
           		{
                   	continue;
           		}

           		NodeExtension *w = v->in_links[i]->src_node;

			if (!(w->enabl))
           	{
				continue;
           	}

           	if (w==target)
           	{
                 continue;
           	}

	   		distw=w->dist;

			distv=(v->dist)+(v->in_links[i]->getWeight()); //Accumulated distance
			current_distance = (v->in_links[i]->getWeight()); //Current distance

			if (w->dist == INFINITY)
           	{
				w->dist = distv;

	      		num_links_to_node=0;
				for (int k=0; k<v->num_in_links; k++)
	      		{
		  		   if((v->in_links[k]->src_node==w)&&(v->in_links[k]->enabl)&&(v->in_links[k]->getWeight()==current_distance))num_links_to_node++;
	       		}
 	      		w->num_paths_m=num_links_to_node*(v->num_paths_m);
              	if(v==target)w->num_paths_m=num_links_to_node*1;

              	w->node_paths_m.resize(w->num_paths_m);
	      		for(j=0;j<w->num_paths_m;j++)w->node_paths_m[j]=v;

	      		w->array_links_path.resize(w->num_paths_m);

				paths_accumulated_previous_node=v->num_paths_m;
				if(v==target)paths_accumulated_previous_node=1;
				j=0;
				for(int r=0;r<paths_accumulated_previous_node;r++)
				{
					for (int k=0; k<v->num_in_links; k++)
	      			{
		  				if((v->in_links[k]->src_node==w)&&(v->in_links[k]->enabl)&&(v->in_links[k]->getWeight()==current_distance))
						{
							w->array_links_path[j] = v->in_links[k];
							j++;
						}
	       			}
				}
               	q.push_back(w);
           	}

	   		else if(distw>(distv+0.0000001)) //Precision of 0.1 microseconds
           		{
				w->dist = distv;
	      			num_links_to_node=0;
				for (int k=0; k<v->num_in_links; k++)
	      		{
		  		   if((v->in_links[k]->src_node==w)&&(v->in_links[k]->enabl)&&(v->in_links[k]->getWeight()==current_distance))num_links_to_node++;
	       		}

 	      			w->num_paths_m=num_links_to_node*(v->num_paths_m);
              		if(v==target)w->num_paths_m=num_links_to_node*1;

              			w->node_paths_m.resize(w->num_paths_m);
	       			for(j=0;j<w->num_paths_m;j++)w->node_paths_m[j]=v;

				w->array_links_path.resize(w->num_paths_m);

				paths_accumulated_previous_node=v->num_paths_m;
				if(v==target)paths_accumulated_previous_node=1;
				j=0;
				for(int r=0;r<paths_accumulated_previous_node;r++)
				{
					for (int k=0; k<v->num_in_links; k++)
	      				{
		  				if((v->in_links[k]->src_node==w)&&(v->in_links[k]->enabl)&&(v->in_links[k]->getWeight()==current_distance))
						{
							w->array_links_path[j] = v->in_links[k];
							j++;
						}
	       				}
				}

				q.push_back( w );
           		}

	   		else if((distw>(distv-0.0000001))&&(distw<(distv+0.0000001)))
           	{
              			old=w->num_paths_m;

                      		counter=0;
		      		for(j=0;j<(w->num_paths_m);j++)if((w->node_paths_m[j])==v)counter++;

	      			num_links_to_node=0;
				for (int k=0; k<v->num_in_links; k++)
	      		{
		  		   if((v->in_links[k]->src_node==w)&&(v->in_links[k]->enabl)&&(v->in_links[k]->getWeight()==current_distance))num_links_to_node++;
	       		}

			    if(v==target)w->num_paths_m=num_links_to_node*1;
			    else
			    {
				 	w->num_paths_m=(w->num_paths_m-counter)+(num_links_to_node*v->num_paths_m);

			    }

			    if(old>w->num_paths_m)
			    {
					auxiliar=old;

					for(int k=0;k<auxiliar;k++)
					{
						if(w->node_paths_m[k]==v)
						{
							for(r=k;r<(auxiliar-1);r++)w->node_paths_m[r]=w->node_paths_m[r+1];
							w->node_paths_m[auxiliar-1]=v;
							aux_output=w->array_links_path[k];
							for(r=k;r<(auxiliar-1);r++)w->array_links_path[r]=w->array_links_path[r+1];
							w->array_links_path[auxiliar-1]=aux_output;
							auxiliar--;
							k--;
						}
					}
					w->node_paths_m.resize(w->num_paths_m);
					w->array_links_path.resize(w->num_paths_m);
			    }
				else
				{
					w->node_paths_m.resize(w->num_paths_m);
					w->array_links_path.resize(w->num_paths_m);

	              	for(j=old;j<(w->num_paths_m);j++)w->node_paths_m[j]=v;

					paths_accumulated_previous_node=(int)(w->num_paths_m-old)/(int)num_links_to_node;


					j=old;
					for(int r=0;r<paths_accumulated_previous_node;r++)
					{
						for (int k=0; k<v->num_in_links; k++)
		      			{
			  				if((v->in_links[k]->src_node==w)&&(v->in_links[k]->enabl)&&(v->in_links[k]->getWeight()==current_distance))
							{
								w->array_links_path[j] = v->in_links[k];
								j++;
							}
		       			}
					}

				}

               			q.push_back( w );
           	}
		}
	}



}





/*

void cTopology::extraeTopologiaLogica(double **matriz, int enlaces, Predicate *predicate)
{
//    const char *data[32];
//    int k=0;
//    data[k++] = type1;
//
//    va_list va;
//    va_start(va,type1);
//    while ((data[k++]=va_arg(va, char *))!=NULL);
//    va_end(va);

    extraeRedLogica(matriz, enlaces, selectByPredicate, (void *)predicate);
}




void cTopology::extraeRedLogica( double **matriz,int enlaces, bool (*selfunc)(cModule *,void *), void *data)
{
    clear();

    int mod_id;
    int i;





    Node *temp_nodev = new Node[simulation.getLastModuleId()];

    // Loop through all modules and find those which have the required
    // parameter with the (optionally) required value.
    int k=0;
    for (mod_id=0; mod_id<=simulation.getLastModuleId(); mod_id++)
    {
        cModule *mod = simulation.getModule(mod_id);
        if (mod && selfunc(mod,data))
        {
            // ith module is OK, insert into nodev[]
            temp_nodev[k].module_id = mod_id;
            temp_nodev[k].wgt = 0;
            temp_nodev[k].enabl = true;

            // init auxiliary variables
            temp_nodev[k].known = 0;
            temp_nodev[k].dist = INFINITY;
            temp_nodev[k].out_path.resize(0);
	    temp_nodev[k].num_caminos = 0;

            // create in_links[] arrays (big enough...)
            temp_nodev[k].num_in_links = 0;
            temp_nodev[k].in_links = new cTopology::Link *[mod->gateCount()];

            k++;
        }
    }
    num_nodes = k;

    int estado_link[num_nodes][2];

    for(i=0;i<num_nodes;i++)
    {
        for(k=0;k<2;k++)estado_link[i][k]=0;
    }

    int num_enlaces_nodo[num_nodes][2];

    for(i=0;i<num_nodes;i++)
    {

        for(k=0;k<2;k++)num_enlaces_nodo[i][k]=0;

    }

    for(i=0;i<enlaces;i++)
    {
        num_enlaces_nodo[(int)matriz[i][0]][0]++;
        num_enlaces_nodo[(int)matriz[i][1]][1]++;
    }



    nodev = new Node[num_nodes];

    memcpy(nodev,temp_nodev,num_nodes*sizeof(Node));
    delete [] temp_nodev;


    // Discover out neighbors too.
    for (k=0; k<num_nodes; k++)
    {
        // Loop through all its gates and find those which come
        // from or go to modules included in the topology.


        cTopology::Link *temp_out_links = new cTopology::Link[num_enlaces_nodo[k][0]];

        int n_out=0;

        for(i=0;i<enlaces;i++)
        {
                if(k==matriz[i][0])
                {
                        temp_out_links[n_out].src_node = nodev+k;
                        temp_out_links[n_out].src_gate = estado_link[k][0];
                        estado_link[k][0]++;
                        temp_out_links[n_out].dest_node = nodev+(int)matriz[i][1];
                        temp_out_links[n_out].dest_gate = estado_link[(int)matriz[i][1]][1];
                        estado_link[(int)matriz[i][1]][1]++;
                        temp_out_links[n_out].wgt = 1.0;
                        temp_out_links[n_out].enabl = true;
                        n_out++;
                }
        }

        nodev[k].num_out_links = n_out;

        nodev[k].out_links = new cTopology::Link[n_out];
        memcpy(nodev[k].out_links,temp_out_links,n_out*sizeof(cTopology::Link));
        delete [] temp_out_links;
    }


    // fill in_links[] arrays
    for (k=0; k<num_nodes; k++)
    {
        for (int l=0; l<nodev[k].num_out_links; l++)
        {
            cTopology::Link *link = &nodev[k].out_links[l];
            link->dest_node->in_links[link->dest_node->num_in_links++] = link;
        }
    }

}



*/







void cTopologyExtension::extractWeigths(const std::vector<std::string>& nedTypeNames)
{
    extractFromWeightedNetwork(selectByNedTypeName, (void *)&nedTypeNames);
}






void cTopologyExtension::extractFromWeightedNetwork(bool (*selfunc)(cModule *,void *), void *data)
{
    clear();

    int mod_id;



    NodeExtension *temp_nodev = new NodeExtension[simulation.getLastModuleId()];

    // Loop through all modules and find those which have the required
    // parameter with the (optionally) required value.
    int k=0;
    for (mod_id=0; mod_id<=simulation.getLastModuleId(); mod_id++)
    {
        cModule *mod = simulation.getModule(mod_id);
        if (mod && selfunc(mod,data))
        {
            // ith module is OK, insert into nodev[]
            temp_nodev[k].module_id = mod_id;
            temp_nodev[k].wgt = 0;
            temp_nodev[k].enabl = true;
            temp_nodev[k].numberEDFAS=0;

            // init auxiliary variables
            temp_nodev[k].known = 0;
            temp_nodev[k].dist = INFINITY;
            temp_nodev[k].out_path=NULL;
            temp_nodev[k].num_paths_m = 0;

            // create in_links[] arrays (big enough...)
            temp_nodev[k].num_in_links = 0;
            temp_nodev[k].in_links = new cTopologyExtension::LinkExtension *[mod->gateCount()];
            k++;
        }
    }
    num_nodes = k;

    nodev = new NodeExtension[num_nodes];
    memcpy(nodev,temp_nodev,num_nodes*sizeof(NodeExtension));
    delete [] temp_nodev;

    // Discover out neighbors too.
    for (k=0; k<num_nodes; k++)
    {
        // Loop through all its gates and find those which come
        // from or go to modules included in the topology.

        cModule *mod = simulation.getModule(nodev[k].module_id);
        cTopologyExtension::LinkExtension *temp_out_links = new cTopologyExtension::LinkExtension[mod->gateCount()];

        int n_out=0;
        for (cModule::GateIterator i(mod); !i.end(); i++)
        {
            cGate *gate = i();
            if (gate->getType()!=cGate::OUTPUT)
            	continue;



            // follow path
            cGate *src_gate = gate;
            do {
                gate = gate->getNextGate();
				}
			while(gate && !selfunc(gate->getOwnerModule(),data));

            // if we arrived in a module in the topology, record it.
            if (gate)
            {
                temp_out_links[n_out].src_node = nodev+k;
                temp_out_links[n_out].src_gate = src_gate->getId();
                temp_out_links[n_out].dest_node = getNodeFor(gate->getOwnerModule());
                temp_out_links[n_out].dest_gate = gate->getId();
               // temp_out_links[n_out].wgt =SIMTIME_DBL(check_and_cast<cDatarateChannel*>(src_gate->getChannel())->getDelay());
                temp_out_links[n_out].wgt=SIMTIME_DBL(check_and_cast<cDelayLengthChannel*>(check_and_cast<cDatarateChannel*>((src_gate->getChannel())))->getDelay());
             //   printf("numero edfas %i\n",temp_out_links[n_out].numberEDFAS);
                temp_out_links[n_out].enabl = true;
                n_out++;
            }
        }
        nodev[k].num_out_links = n_out;

        nodev[k].out_links = new cTopologyExtension::LinkExtension[n_out];
        memcpy(nodev[k].out_links,temp_out_links,n_out*sizeof(cTopologyExtension::LinkExtension));
        delete [] temp_out_links;
    }

    // fill in_links[] arrays
    for (k=0; k<num_nodes; k++)
    {
        for (int l=0; l<nodev[k].num_out_links; l++)
        {
            cTopologyExtension::LinkExtension *link = &nodev[k].out_links[l];
            link->dest_node->in_links[link->dest_node->num_in_links++] = link;
        }
    }

}


void cTopologyExtension::NoWavelengthsConsideration()
{
    for (int k=0; k<num_nodes; k++)
    {
	cModule *mod = simulation.getModule(nodev[k].module_id);

	for (cModule::GateIterator i(mod); !i.end(); i++)
        {
        	cGate *gate = i();

		for (int l=0; l<nodev[k].num_out_links; l++)
        	{
			if(((nodev[k].out_links[l].src_gate)==(gate->getId()))&&(gate->getIndex()!=0))
			{
				nodev[k].out_links[l].enabl = false;
				l=nodev[k].num_out_links;
			}
        	}
    	}
   }
}



void cTopologyExtension::virtualExtractByNedTypeName(const std::vector<std::string>& nedTypeNames, vector < vector < int> > virtualTopology, int num_virtual_link)
{
    virtualExtractFromNetwork(virtualTopology,num_virtual_link, selectByNedTypeName, (void *)&nedTypeNames);
}



void cTopologyExtension::virtualExtractFromNetwork(vector < vector < int> > virtualTopology,int num_virtual_link, bool (*selfunc)(cModule *,void *), void *data)
{
    clear();



    NodeExtension *temp_nodev = new NodeExtension[simulation.getLastModuleId()];

    // Loop through all modules and find those which have the required
    // parameter with the (optionally) required value.
    int i;
    int k=0;
    for (int mod_id=0; mod_id<=simulation.getLastModuleId(); mod_id++)
    {
        cModule *mod = simulation.getModule(mod_id);
        if (mod && selfunc(mod,data))
        {
            // ith module is OK, insert into nodev[]
            temp_nodev[k].module_id = mod_id;
            temp_nodev[k].wgt = 0;
            temp_nodev[k].enabl = true;

            // init auxiliary variables
            temp_nodev[k].known = 0;
            temp_nodev[k].dist = INFINITY;
            //temp_nodev[k].out_path.resize(0);
            temp_nodev[k].out_path=NULL;

            // create in_links[] arrays (big enough...)
            temp_nodev[k].num_in_links = 0;
            temp_nodev[k].in_links = new cTopologyExtension::LinkExtension *[mod->gateCount()];
            k++;
        }
    }
    num_nodes = k;

    nodev = new NodeExtension[num_nodes];
    memcpy(nodev, temp_nodev, num_nodes*sizeof(NodeExtension));
    delete [] temp_nodev;


    int estado_link[num_nodes][2];
    int num_enlaces_nodo[num_nodes][2];

    for(i=0;i<num_nodes;i++)
    {
    	for(k=0;k<2;k++)
    		{
				estado_link[i][k]=0;
				num_enlaces_nodo[i][k]=0;
    		}
    }

    for(i=0;i<num_virtual_link;i++)
    {
        num_enlaces_nodo[(int)virtualTopology[i][0]][0]++;
        num_enlaces_nodo[(int)virtualTopology[i][1]][1]++;
    }


    // Discover out neighbors too.
    for (int k=0; k<num_nodes; k++)
    {
        // Loop through all its gates and find those which come
        // from or go to modules included in the topology.

        cTopologyExtension::LinkExtension *temp_out_links = new cTopologyExtension::LinkExtension[num_enlaces_nodo[k][0]];

        int n_out=0;


        for(i=0;i<num_virtual_link;i++)
        {
			if(k==virtualTopology[i][0])
			{
				temp_out_links[n_out].src_node = nodev+k;
				temp_out_links[n_out].src_gate = estado_link[k][0];
				estado_link[k][0]++;
				temp_out_links[n_out].dest_node = nodev+(int)virtualTopology[i][1];
				temp_out_links[n_out].dest_gate = estado_link[(int)virtualTopology[i][1]][1];
				estado_link[(int)virtualTopology[i][1]][1]++;
				temp_out_links[n_out].wgt = 1.0;
				temp_out_links[n_out].enabl = true;
				n_out++;
			}
        }

        nodev[k].num_out_links = n_out;

        nodev[k].out_links = new cTopologyExtension::LinkExtension[n_out];

        memcpy(nodev[k].out_links, temp_out_links, n_out*sizeof(cTopologyExtension::LinkExtension));
        delete [] temp_out_links;


    }

    // fill in_links[] arrays
    for (int k=0; k<num_nodes; k++)
    {
        for (int l=0; l<nodev[k].num_out_links; l++)
        {
            cTopologyExtension::LinkExtension *link = &nodev[k].out_links[l];

            link->dest_node->in_links[link->dest_node->num_in_links++] = link;
        }
    }
}








